"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XrpTransaction = void 0;
const types_1 = require("../../types");
const constants_1 = require("../../utils/constants");
const utils_1 = require("../../utils/utils");
const TransactionBase_1 = require("../TransactionBase");
class XrpTransaction extends TransactionBase_1.TransactionBase {
    get txid() {
        return this.hash;
    }
    get stdTxid() {
        return this.txid;
    }
    get hash() {
        return this.data.result.hash;
    }
    get reference() {
        if (this.data.result.Memos) {
            return this.data.result.Memos.map((memoobj) => {
                return memoobj.Memo.MemoData || "";
            });
        }
        return [];
    }
    get stdPaymentReference() {
        let paymentReference = this.reference.length === 1 ? (0, utils_1.prefix0x)(this.reference[0]) : "";
        if (!(0, utils_1.isValidBytes32Hex)(paymentReference)) {
            paymentReference = utils_1.ZERO_BYTES_32;
        }
        return paymentReference;
    }
    get unixTimestamp() {
        return (constants_1.XRP_UTD +
            // @ts-ignore
            this.data.result.date);
    }
    get sourceAddresses() {
        return [this.data.result.Account];
    }
    get receivingAddresses() {
        if (this.data.result.TransactionType == "Payment") {
            let payment = this.data.result;
            return [payment.Destination];
        }
        // TODO: Check if in other types of payments one has something similar to Destination
        return [];
    }
    get fee() {
        if (!this.data.result.Fee) {
            return (0, utils_1.toBN)(0);
        }
        return (0, utils_1.toBN)(this.data.result.Fee);
    }
    get spentAmounts() {
        if (this.isNativePayment) {
            let payment = this.data.result;
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: (0, utils_1.toBN)(payment.Amount).add(this.fee),
                },
            ];
        }
        // TODO Check for other non-native payment types
        // TODO: Check whether in some other non-payment cases spent amount is different
        return [
            {
                amount: (0, utils_1.toBN)(this.fee),
            },
        ];
    }
    get receivedAmounts() {
        let metaData = this.data.result.meta || this.data.result.metaData;
        if (this.isNativePayment) {
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(metaData.delivered_amount),
                },
            ];
        }
        return [];
    }
    get type() {
        return this.data.result.TransactionType;
        // Possible types available at Transaction object in xrpl lib
    }
    get isNativePayment() {
        return this.currencyName === constants_1.XRP_NATIVE_TOKEN_NAME;
    }
    get currencyName() {
        // With ripple this is currency code
        if (this.type === "Payment") {
            if (this.data.result.Amount.currency) {
                return this.data.result.Amount.currency;
            }
            return constants_1.XRP_NATIVE_TOKEN_NAME;
        }
        // TODO Check for other types of transactions
        return "";
    }
    get elementaryUnits() {
        // TODO this is dependent on currency we are using
        return (0, utils_1.toBN)(constants_1.XRP_MDU);
    }
    get successStatus() {
        // https://xrpl.org/transaction-results.html
        let metaData = this.data.result.meta || this.data.result.metaData;
        let result = metaData.TransactionResult;
        if (result === "tesSUCCESS") {
            // https://xrpl.org/tes-success.html
            return types_1.TransactionSuccessStatus.SUCCESS;
        }
        if (result.startsWith("tec")) {
            // https://xrpl.org/tec-codes.html
            switch (result) {
                case "tecDST_TAG_NEEDED":
                case "tecNO_DST":
                case "tecNO_DST_INSUF_XRP":
                case "tecNO_PERMISSION":
                    return types_1.TransactionSuccessStatus.RECEIVER_FAILURE;
                default:
                    return types_1.TransactionSuccessStatus.SENDER_FAILURE;
            }
        }
        // Other codes: tef, tel, tem, ter are not applied to ledgers
        return types_1.TransactionSuccessStatus.SENDER_FAILURE;
    }
    paymentSummary(client, inUtxo, utxo, makeFullPayment) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isNativePayment) {
                if (this.type === "Payment") {
                    // token transfer
                    const value = this.data.result.Amount.value;
                    const valueSplit = value.split(".");
                    let eleUnits = 1;
                    if (valueSplit.length === 2) {
                        eleUnits = Math.pow(10, valueSplit[1].length);
                    }
                    return {
                        isNativePayment: false,
                        isTokenTransfer: true,
                        tokenElementaryUnits: (0, utils_1.toBN)(eleUnits),
                        receivedTokenAmount: (0, utils_1.toBN)(valueSplit.join("")),
                        tokenName: this.currencyName,
                    };
                }
                else {
                    return { isNativePayment: false };
                }
            }
            return {
                isNativePayment: true,
                sourceAddress: this.sourceAddresses[0],
                receivingAddress: this.receivingAddresses[0],
                spentAmount: this.spentAmounts[0].amount,
                // TODO: Check if intended sent value can be set
                receivedAmount: this.successStatus === types_1.TransactionSuccessStatus.SUCCESS ? this.receivedAmounts[0].amount : (0, utils_1.toBN)(0),
                oneToOne: true,
                paymentReference: this.stdPaymentReference,
                isFull: true,
            };
        });
    }
    //////////////////////////////
    //// Xrp specific methods ////
    //////////////////////////////
    get isAccountCreate() {
        if (this.type === "Payment") {
            if (this.data.result.meta) {
                if (typeof this.data.result.meta === "string") {
                    return false;
                }
                const Meta = this.data.result.meta;
                for (let elem of Meta.AffectedNodes) {
                    if ("CreatedNode" in elem) {
                        if ("NewFields" in elem.CreatedNode) {
                            if ("Account" in elem.CreatedNode.NewFields) {
                                if (elem.CreatedNode.NewFields.Account === this.receivingAddresses[0]) {
                                    return true;
                                }
                            }
                        }
                    }
                }
                return false;
            }
            return false;
        }
        return false;
    }
}
exports.XrpTransaction = XrpTransaction;
//# sourceMappingURL=XrpTransaction.js.map